@@ -24,10 +24,11 @@ from sqlalchemy.orm.collections import attribute_mapped_collection
 from sqlalchemy.orm import exc
 
 from neutron_lib import constants as const
+from neutron_lib.db import api as db_api
 from neutron_lib.db import model_base
+from neutron_lib.db import model_query
+from neutron_lib.db import utils as db_utils
 
-from neutron.db import api as db_api
-from neutron.db import common_db_mixin
 from neutron.db import models_v2
 
 from networking_sfc.extensions import flowclassifier as fc_ext
@@ -73,8 +74,7 @@ class FlowClassifier(model_base.BASEV2, model_base.HasId,
         cascade='all, delete-orphan')
 
 
-class FlowClassifierDbPlugin(fc_ext.FlowClassifierPluginBase,
-                             common_db_mixin.CommonDbMixin):
+class FlowClassifierDbPlugin(fc_ext.FlowClassifierPluginBase):
 
     @classmethod
     def _check_port_range_valid(cls, port_range_min,
@@ -230,12 +230,12 @@ class FlowClassifierDbPlugin(fc_ext.FlowClassifierPluginBase,
         self._check_ip_prefix_valid(destination_ip_prefix, ethertype)
         logical_source_port = fc['logical_source_port']
         logical_destination_port = fc['logical_destination_port']
-        with db_api.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             if logical_source_port is not None:
                 self._get_port(context, logical_source_port)
             if logical_destination_port is not None:
                 self._get_port(context, logical_destination_port)
-            query = self._model_query(context, FlowClassifier)
+            query = model_query.query_with_hooks(context, FlowClassifier)
             for flow_classifier_db in query.all():
                 if self.flowclassifier_conflict(
                     fc,
@@ -289,43 +289,46 @@ class FlowClassifierDbPlugin(fc_ext.FlowClassifierPluginBase,
                 for k, param in flow_classifier.l7_parameters.items()
             }
         }
-        return self._fields(res, fields)
+        return db_utils.resource_fields(res, fields)
 
     @log_helpers.log_method_call
+    @db_api.CONTEXT_READER
     def get_flow_classifiers(self, context, filters=None, fields=None,
                              sorts=None, limit=None, marker=None,
                              page_reverse=False):
-        marker_obj = self._get_marker_obj(context, 'flow_classifier',
-                                          limit, marker)
-        return self._get_collection(context,
-                                    FlowClassifier,
-                                    self._make_flow_classifier_dict,
-                                    filters=filters, fields=fields,
-                                    sorts=sorts,
-                                    limit=limit, marker_obj=marker_obj,
-                                    page_reverse=page_reverse)
+        marker_obj = db_utils.get_marker_obj(self, context, 'flow_classifier',
+                                             limit, marker)
+        return model_query.get_collection(
+            context,
+            FlowClassifier,
+            self._make_flow_classifier_dict,
+            filters=filters, fields=fields,
+            sorts=sorts,
+            limit=limit, marker_obj=marker_obj,
+            page_reverse=page_reverse)
 
     @log_helpers.log_method_call
+    @db_api.CONTEXT_READER
     def get_flow_classifier(self, context, id, fields=None):
         flow_classifier = self._get_flow_classifier(context, id)
         return self._make_flow_classifier_dict(flow_classifier, fields)
 
     def _get_flow_classifier(self, context, id):
         try:
-            return self._get_by_id(context, FlowClassifier, id)
-        except exc.NoResultFound:
-            raise fc_ext.FlowClassifierNotFound(id=id)
+            return model_query.get_by_id(context, FlowClassifier, id)
+        except exc.NoResultFound as no_res_found:
+            raise fc_ext.FlowClassifierNotFound(id=id) from no_res_found
 
     def _get_port(self, context, id):
         try:
-            return self._get_by_id(context, models_v2.Port, id)
-        except exc.NoResultFound:
-            raise fc_ext.FlowClassifierPortNotFound(id=id)
+            return model_query.get_by_id(context, models_v2.Port, id)
+        except exc.NoResultFound as no_res_found:
+            raise fc_ext.FlowClassifierPortNotFound(id=id) from no_res_found
 
     @log_helpers.log_method_call
     def update_flow_classifier(self, context, id, flow_classifier):
         new_fc = flow_classifier['flow_classifier']
-        with db_api.context_manager.writer.using(context):
+        with db_api.CONTEXT_WRITER.using(context):
             old_fc = self._get_flow_classifier(context, id)
             old_fc.update(new_fc)
             return self._make_flow_classifier_dict(old_fc)
@@ -333,10 +336,10 @@ class FlowClassifierDbPlugin(fc_ext.FlowClassifierPluginBase,
     @log_helpers.log_method_call
     def delete_flow_classifier(self, context, id):
         try:
-            with db_api.context_manager.writer.using(context):
+            with db_api.CONTEXT_WRITER.using(context):
                 fc = self._get_flow_classifier(context, id)
                 context.session.delete(fc)
-        except AssertionError:
-            raise fc_ext.FlowClassifierInUse(id=id)
+        except AssertionError as exc:
+            raise fc_ext.FlowClassifierInUse(id=id) from exc
         except fc_ext.FlowClassifierNotFound:
             LOG.info("Deleting a non-existing flow classifier.")
